package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.data.Record;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.wecom.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.wecom.dao.mapper.UserMapper;
import com.tzld.piaoquan.wecom.model.bo.PushMessage;
import com.tzld.piaoquan.wecom.model.po.MessageAttachment;
import com.tzld.piaoquan.wecom.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.wecom.model.po.User;
import com.tzld.piaoquan.wecom.model.po.UserExample;
import com.tzld.piaoquan.wecom.service.MessageService;
import com.tzld.piaoquan.wecom.utils.DateUtil;
import com.tzld.piaoquan.wecom.utils.MessageUtil;
import com.tzld.piaoquan.wecom.utils.OdpsUtil;
import com.tzld.piaoquan.wecom.utils.page.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.GUARANTEED_MINIPROGRAM_KEY;

@Component
public class WeComMessageDataJob {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private MessageService messageService;

    private static final int sum = 3;

    List<PushMessage> goodList = new ArrayList<>();
    List<Long> saveList = new ArrayList<>();

    //初始化操作
    void init() {
        String sql = String.format("SELECT * FROM loghubods.history_good_video_can_push_user_list where dt = %s;", DateUtil.getBeforeDayDateString());
        List<Record> recordList = OdpsUtil.getOdpsData(sql);
        if (CollectionUtils.isEmpty(recordList)) {
            return;
        }
        List<PushMessage> list = new ArrayList<>();
        for (Record record : recordList) {
            PushMessage pushMessage = new PushMessage();
            Long videoId = Long.parseLong((String) record.get(0));
            Set<Long> userIds = new HashSet<>(JSONObject.parseArray((String) record.get(1), Long.class));
            pushMessage.setVideoId(videoId);
            pushMessage.setUserIds(userIds);
            list.add(pushMessage);
        }
        goodList = list;
        List<Long> saveVideoIds = Objects.requireNonNull(redisTemplate.opsForList().range(GUARANTEED_MINIPROGRAM_KEY, 0, 2))
                .stream().map(o -> (Integer) o).map(String::valueOf).map(Long::parseLong).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(saveVideoIds)) {
            throw new RuntimeException("保底数据为空");
        }
        MessageAttachmentExample example = new MessageAttachmentExample();
        example.createCriteria().andMiniprogramVideoIdIn(saveVideoIds);
        List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(messageAttachmentList) || messageAttachmentList.size() < saveVideoIds.size()) {
            throw new RuntimeException("保底数据获取失败");
        }
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            if (messageAttachment.getUpdateTime().compareTo(DateUtil.getThatDayDate()) < 0) {
                throw new RuntimeException("保底数据不是最新的");
            }
        }
        saveList = saveVideoIds;
    }

    public void assemble() {
        init();
        Map<String, List<String>> res = new HashMap<>();
        UserExample example = new UserExample();
        long count = userMapper.countByExample(example);
        int page = 1;
        int pageSize = 1000;
        long totalPageSize = count / pageSize + 1;
        for (; page <= totalPageSize; page++) {
            example.setPage(new Page<>(page, pageSize));
            List<User> userList = userMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(userList)) {
                continue;
            }
            for (User user : userList) {
                String pushKey = getPushKey(user);
                if (res.containsKey(pushKey)) {
                    res.get(pushKey).add(user.getExternalUserId3rdParty());
                } else {
                    res.put(pushKey, Lists.newArrayList(user.getExternalUserId3rdParty()));
                }
            }
        }
        pushMessage(res);
    }


    public String getPushKey(User user) {
        int n = 0;
        StringBuilder key = new StringBuilder();
        for (PushMessage pushMessage : goodList) {
            if (pushMessage.getUserIds().contains(user.getId())) {
                if (key.length() == 0) {
                    key.append(pushMessage.getVideoId());
                    n++;
                } else {
                    key.append("_");
                    key.append(pushMessage.getVideoId());
                }
                n++;
                if (n > sum) {
                    return key.toString();
                }
            }
        }
        for (Long videoId : saveList) {
            if (key.length() == 0) {
                key.append(videoId);
                n++;
            } else {
                key.append("_");
                key.append(videoId);
            }
            n++;
            if (n > sum) {
                return key.toString();
            }
        }
        return key.toString();
    }

    public void pushMessage(Map<String, List<String>> map) {
        List<JSONObject> pushList = new ArrayList<>();
        //发送人员  暂时写死 只有两个
        List<String> staffExtIdList = new ArrayList<>();
        staffExtIdList.add("wogizUDQAAk5Z_5nLRD-XTAPU9poaRCw");
        staffExtIdList.add("wogizUDQAAzM5HbqIyA4k3KR6D8vKp3w");
        String text = messageService.getMessageText();
        String name = MessageUtil.getName();
        for (Map.Entry<String, List<String>> entry : map.entrySet()) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("name", name);
            jsonObject.put("text", text);
            JSONArray attachments = new JSONArray();
            List<Long> videoIdList = Arrays.stream(entry.getKey().split("_")).map(Long::parseLong).collect(Collectors.toList());
            for (Long videoId : videoIdList) {
                JSONObject attachment = new JSONObject();
                attachment.put("msgtype", "miniprogram");
                MessageAttachmentExample example = new MessageAttachmentExample();
                example.createCriteria().andMiniprogramVideoIdEqualTo(videoId);
                List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
                if (CollectionUtils.isEmpty(messageAttachmentList)) {
                    throw new RuntimeException("附件信息查询异常");
                }
                MessageAttachment messageAttachment = messageAttachmentList.get(0);
                JSONObject miniprogram = new JSONObject();
                miniprogram.put("appid", messageAttachment.getAppid());
                miniprogram.put("title", messageAttachment.getTitle());
                miniprogram.put("page", messageAttachment.getPage());
                miniprogram.put("cover", messageAttachment.getCover());
                attachment.put("miniprogram", miniprogram);
                attachments.add(attachment);
            }
            jsonObject.put("attachments", attachments);
            List<List<String>> lists = Lists.partition(entry.getValue(), 10000);
            for (List<String> list : lists) {
                List<JSONObject> staffEuList = new ArrayList<>();
                JSONObject newJSONObject = new JSONObject();
                newJSONObject.putAll(jsonObject);
                for (String staffExtId : staffExtIdList) {
                    JSONObject staff_eu = new JSONObject();
                    staff_eu.put("staff_ext_id", staffExtId);
                    staff_eu.put("eu_ext_ids", list);
                    staffEuList.add(staff_eu);
                }
                newJSONObject.put("staff_eu_list", staffEuList);
                pushList.add(newJSONObject);
            }
        }
        if (CollectionUtils.isEmpty(pushList)) {
            throw new RuntimeException("推送视频生成失败");
        }
        for (JSONObject jsonObject : pushList) {
            messageService.pushMessage(jsonObject);
        }
    }
}
