package com.tzld.piaoquan.wecom.service.Impl;

import com.tzld.piaoquan.wecom.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.wecom.common.exception.CustomizeException;
import com.tzld.piaoquan.wecom.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.wecom.model.po.MessageAttachment;
import com.tzld.piaoquan.wecom.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.wecom.model.vo.MiniprogramVo;
import com.tzld.piaoquan.wecom.service.MessageAttachmentService;
import com.tzld.piaoquan.wecom.utils.MessageUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static com.tzld.piaoquan.wecom.common.constant.MessageConstant.appid;
import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.GUARANTEED_MINIPROGRAM_KEY;
import static com.tzld.piaoquan.wecom.common.enums.ExceptionCodeEnum.PARAMS_ERROR;


@Service
public class MessageAttachmentServiceImpl implements MessageAttachmentService {


    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public void addMiniprogram(List<MessageAttachment> messageAttachmentList) {
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria()
                    .andAppidEqualTo(messageAttachment.getAppid())
                    .andTypeEqualTo(messageAttachment.getType())
                    .andMiniprogramVideoIdEqualTo(messageAttachment.getMiniprogramVideoId());
            List<MessageAttachment> messageAttachments = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachments)) {
                messageAttachmentMapper.insert(messageAttachment);
                continue;
            }
            messageAttachment.setId(messageAttachments.get(0).getId());
            messageAttachmentMapper.updateByPrimaryKeySelective(messageAttachment);
        }
    }

    @Override
    public void createGuaranteedMiniprogram(List<MiniprogramVo> miniprogramVoList) {
        List<MessageAttachment> messageAttachmentList = new ArrayList<>();
        List<Long> videoList = new ArrayList<>();
        for (MiniprogramVo miniprogramVo : miniprogramVoList) {
            if (StringUtils.isEmpty(miniprogramVo.getPage())) {
                throw new CustomizeException(PARAMS_ERROR);
            }
            if (StringUtils.isEmpty(miniprogramVo.getCover())) {
                throw new CustomizeException(PARAMS_ERROR);
            }
            if (StringUtils.isEmpty(miniprogramVo.getTitle())) {
                throw new CustomizeException(PARAMS_ERROR);
            }
            Long videoId = MessageUtil.getVideoId(miniprogramVo.getPage());
            if (videoId == null) {
                throw new CustomizeException(PARAMS_ERROR);
            }
            if (StringUtils.isEmpty(miniprogramVo.getAppid())) {
                miniprogramVo.setAppid(appid);
            }
            MessageAttachment messageAttachment = new MessageAttachment();
            BeanUtils.copyProperties(miniprogramVo, messageAttachment);
            messageAttachment.setMiniprogramVideoId(videoId);
            messageAttachment.setType(MessageAttachmentTypeEnum.MINIPROGRAM.getType());
            messageAttachmentList.add(messageAttachment);
            videoList.add(videoId);
        }
        addMiniprogram(messageAttachmentList);
        redisTemplate.delete(GUARANTEED_MINIPROGRAM_KEY);
        Collections.reverse(videoList);
        for (Long video : videoList) {
            redisTemplate.opsForList().leftPush(GUARANTEED_MINIPROGRAM_KEY, video);
        }
    }
}
